class Pizza < ActiveRecord::Base
  belongs_to :crust_type
  belongs_to :user
  has_many :pizza_toppings
  has_many :toppings, :through => :pizza_toppings
  
  attr_accessible :name, :crispy, :crust_type_id, :topping_ids
  
  validates_presence_of :name, :message => "^Your pizza must have a name"
  validate :no_crispy_with_deep_dish
  validate :too_many_toppings
  
  def description
    descriptions = []
    descriptions << "#{crust_type.name} Crust" if crust_type
    descriptions << "Extra Crispy" if crispy?
    if toppings.size > 3
      descriptions << "#{toppings.size} Toppings"
    else
      descriptions += toppings.map(&:name)
    end
    descriptions.join(', ')
  end
  
  private
  
  def no_crispy_with_deep_dish
    if crispy? && crust_type && crust_type.name =~ /Deep/
      errors.add :crispy, "^Extra crispy isn't available for deep dish style crust."
    end
  end
  
  def too_many_toppings
    if toppings.size > 7
      errors.add_to_base "Can't have more than 7 toppings on your pizza."
    end
  end
end
